@@ -1,17 +1,14 @@
 import numpy as np
-import numpy.ma as ma
-from copy import deepcopy,copy
 
-import matplotlib.cm as mcm
-#from matplotlib import ticker
 from matplotlib.ticker import MultipleLocator
+#from cycler import cycler
 
 import astropy.units as u
 from astropy.nddata import StdDevUncertainty
 
 from ..measurement import Measurement
 from .plotbase import PlotBase
-from ..pdrutils import float_formatter,LOGE,is_odd,to
+from ..pdrutils import float_formatter,LOGE
 
 class ExcitationPlot(PlotBase):
     """
@@ -22,6 +19,17 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
         self._xlim = []
         self._ylim = []
         self._label = label
+        self._logfile = None
+
+    def _sorted_by_vibrational_level(self,measurements):
+        # d is a dict of measurements
+        ret = dict()
+        for m in measurements:
+            _key=f'v={self._tool._ac.loc[m]["vu"]}'
+            if _key not in ret: 
+                ret[_key] = []
+            ret[_key].append(measurements[m])
+        return ret
 
     def ex_diagram(self,position=None,size=None,norm=True,show_fit=False,**kwargs):
         #@todo position and size might not necessarily match how the fit was done.
@@ -40,21 +48,28 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
         :type show_fit: bool
         """
         kwargs_opts = {'xmin':0.0,
-                      'xmax':5000.0, #@TODO this should scale with max(energy)
+                      'xmax':None, #  we use np.max() later if user does not specify
                       'ymax':22,
                       'ymin': 15,
+                      'xlabel':None,
+                      'ylabel':None,
                       'grid' :False,
                       'figsize':(10,7),
                       'capsize':3,
                       'linewidth': 2.0,
                       'markersize': 8,
                       'color':None,
-                      'axis':None}
+                      'axis':None,
+                      'label':None,
+                      'aspect': 'auto',
+                      'bbox_to_anchor':None,
+                      'loc':"best",
+                      'test': False}
         kwargs_opts.update(kwargs)
 
-        cdavg = self._tool.average_column_density(norm=norm, position=position, size=size, line=False)
-        energies = self._tool.energies(line=False)
-        energy = np.array([c for c in energies.values()])
+        cdavg = self._tool.average_column_density(norm=norm, position=position, size=size, line=True)
+        energies = self._tool.energies(line=True)
+        energy = np.array(list(energies.values()))
         colden = np.squeeze(np.array([c.data for c in cdavg.values()]))
         error = np.squeeze(np.array([c.error for c in cdavg.values()]))
         sigma = LOGE*error/colden
@@ -63,14 +78,39 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
             _axis = self._axis
         else:
             _axis = kwargs_opts['axis']
-        if self._tool.opr_fitted and show_fit:
-            _label = "LTE"
+        if kwargs_opts['label'] != 'v':
+            if self._tool.opr_fitted and show_fit:
+                _label = "LTE"
+            else:
+                _label = '$'+self._label+'$ data'
+            ec = _axis.errorbar(energy,np.log10(colden),yerr=sigma,
+                                fmt="o", capsize=kwargs_opts['capsize'],
+                                label=_label, lw=kwargs_opts['linewidth'],
+                                ms=kwargs_opts['markersize'], 
+                                color=kwargs_opts['color'])
         else:
-            _label = '$'+self._label+'$ data'
-        ec = _axis.errorbar(energy,np.log10(colden),yerr=sigma,
-                            fmt="o", capsize=kwargs_opts['capsize'],
-                            label=_label, lw=kwargs_opts['linewidth'],
-                            ms=kwargs_opts['markersize'],color=kwargs_opts['color'])
+            # return dict of arrays of measuremtents with keys v=0,v=1,v=2 etc
+            cdsort = self._sorted_by_vibrational_level(cdavg)
+            ensort = self._sorted_by_vibrational_level(energies)
+            #print("ENSORT" ,ensort.values())
+            #cyc = cycler('color',  self._CB_color_cycle)
+            #cyfill = cycler('fillstyle',['full', 'none', 'full', 'none', 'full', 'none', 'full', 'none', 'full'])
+            #self._plt.rc('axes', prop_cycle=(cyc+cyfill))
+
+            fmtd = {False: 'o',True:'^'} #there is no cycler for fmt, do it manually
+            fmtb=False
+            for key in cdsort:
+                cs = np.squeeze(np.array([m.value[0] for m in cdsort[key]]))
+                es = np.squeeze(np.array([m.error[0] for m in cdsort[key]]))
+                ens =  np.array([c for c in ensort[key]])
+                #print(f"LOG10(CD({key}))={np.log10(cs)}")
+                #print(f"E{key} = {ens}")
+                sigma = LOGE*es/cs
+                ec = _axis.errorbar(ens,np.log10(cs),yerr=sigma,
+                    fmt=fmtd[fmtb], capsize=kwargs_opts['capsize'],
+                    label=key, lw=kwargs_opts['linewidth'],
+                    ms=kwargs_opts['markersize'])
+                #fmtb = not fmtb
         tt = self._tool
         if self._tool.opr_fitted and show_fit:
             if position is not None and len(np.shape(tt.opr))>1:
@@ -82,7 +122,7 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
                 opr_p = tt.opr
             cddn = colden*self._tool._canonical_opr/opr_p
             # Plot only the odd-J ones scaled by fitted OPR
-            odd_index = np.where([is_odd(c) for c in cdavg.keys()])
+            odd_index = np.where([self._tool._is_ortho(c) for c in cdavg.keys()])
             #color = ec.lines[0].get_color() # want these to be same color as data
             _axis.errorbar(x=energy[odd_index],
                                 y=np.log10(cddn[odd_index]),marker="^",
@@ -92,20 +132,32 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
                                 linestyle='none',color='k',
                                 lw=kwargs_opts['linewidth'],
                                 ms=kwargs_opts['markersize'])
-        _axis.set_xlabel("$E_u/k$ (K)")
-        if norm:
-            _axis.set_ylabel("log $(N_u/g_u) ~({\\rm cm}^{-2})$")
+        if kwargs_opts['xlabel'] is None:
+            _axis.set_xlabel("$E_u/k$ (K)")
         else:
-            _axis.set_ylabel("log $(N_u) ~({\\rm cm}^{-2})$")
-        # label the points with e.g. J=2,3,4...
-        first=True
-        for lab in sorted(cdavg):
-            if first:
-                ss="J="+str(lab)
-                first=False
+            _axis.set_xlabel(kwargs_opts['xlabel'])
+        if kwargs_opts['ylabel'] is None:
+            if norm:
+                _axis.set_ylabel("log $(N_u/g_u) ~({\\rm cm}^{-2})$")
             else:
-                ss=str(lab)
-            _axis.text(x=energies[lab]+100,y=np.log10(cdavg[lab]),s=ss)
+                _axis.set_ylabel("log $(N_u) ~({\\rm cm}^{-2})$")
+        else:
+            _axis.set_ylabel(kwargs_opts['ylabel'])
+        first = True
+        if kwargs_opts['label']=='id':
+            for lab in sorted(cdavg):
+                _axis.text(x=energies[lab]+100,y=np.log10(cdavg[lab]),s=str(lab))
+        elif kwargs_opts['label'] == 'j':  # label the points with e.g. J=2,3,4...
+            for lab in sorted(cdavg):
+                # this fails because the lowest J may not be the first data point.
+                # we'd have to sort on Ju of the data. Which isn't even unique
+                # if there are multiple vibrational levels.
+                #if first:
+                #    ss="J="+str(self._tool._ac.loc[lab]["Ju"])
+                #    first=False
+                #else:
+                ss=str(self._tool._ac.loc[lab]["Ju"])
+                _axis.text(x=energies[lab]+100,y=np.log10(cdavg[lab]),s=ss)
         handles,labels=_axis.get_legend_handles_labels()
         if show_fit:
             if tt.fit_result is None:
@@ -117,12 +169,16 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
             if tt.fit_result[position] is None:
                 raise ValueError(f"The Excitation Tool was unable to fit pixel {position}. Try show_fit=False")
             x_fit = np.linspace(0,max(energy), 30)
+            #@TODO This now depends on tool._numcomponents
             outpar = tt.fit_result[position].params.valuesdict()
-            labcold = r"$T_{cold}=$" + f"{tt.tcold[position]:3.0f}" +r"$\pm$" + f"{tt.tcold.error[position]:.1f} {tt.tcold.unit}"
-            #labcold = r"$T_{cold}=$" + f"{tt.tcold[position]:3.1f}"
-            #labhot= r"$T_{hot}=$" + f"{tt.thot.value:3.0f}"+ r"$\pm$" + f"{tt.thot.error:.1f} {tt.thot.unit}"
-            #labhot= r"$T_{hot}=$" + f"{tt.thot[position]:3.1f}"
-            labhot= r"$T_{hot}=$" + f"{tt.thot[position]:3.0f}"+ r"$\pm$" + f"{tt.thot.error[position]:.1f} {tt.thot.unit}"
+            if tt.numcomponents == 2:
+                labcold = r"$T_{cold}=$" + f"{tt.tcold[position]:3.0f}" +r"$\pm$" + f"{tt.tcold.error[position]:.1f} {tt.tcold.unit}"
+                #labcold = r"$T_{cold}=$" + f"{tt.tcold[position]:3.1f}"
+                #labhot= r"$T_{hot}=$" + f"{tt.thot.value:3.0f}"+ r"$\pm$" + f"{tt.thot.error:.1f} {tt.thot.unit}"
+                #labhot= r"$T_{hot}=$" + f"{tt.thot[position]:3.1f}"
+                labhot= r"$T_{hot}=$" + f"{tt.thot[position]:3.0f}"+ r"$\pm$" + f"{tt.thot.error[position]:.1f} {tt.thot.unit}"
+            elif tt.numcomponents == 1:
+                labcold = r"$T=$" + f"{tt.tcold[position]:3.0f}" +r"$\pm$" + f"{tt.tcold.error[position]:.1f} {tt.tcold.unit}"
             if position == 0:
                 labnh = r"$N("+self._label+")=" + float_formatter(tt.total_colden,2)+"$"
             else:
@@ -130,22 +186,34 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
             _axis.plot(x_fit,tt._one_line(x_fit, outpar['m1'],
                             outpar['n1']), '.' ,label=labcold,
                             lw=kwargs_opts['linewidth'])
-            _axis.plot(x_fit,tt._one_line(x_fit, outpar['m2'],
-                            outpar['n2']), '.', label=labhot,
-                            lw=kwargs_opts['linewidth'])
-
+            if tt.numcomponents == 2:
+                _axis.plot(x_fit,tt._one_line(x_fit, outpar['m2'],
+                                outpar['n2']), '.', label=labhot,
+                                lw=kwargs_opts['linewidth'])
             _axis.plot(x_fit, tt.fit_result[position].eval(x=x_fit,fit_opr=False), label="fit")
             handles,labels=_axis.get_legend_handles_labels()
             #kluge to ensure N(H2) label is last
             phantom = _axis.plot([],marker="", markersize=0,ls="",lw=0)
             handles.append(phantom[0])
             labels.append(labnh)
-
+        # Scale xaxis with max(energy). Round up to nearest 1000
+        if kwargs_opts['xmax'] is None:
+            kwargs_opts['xmax'] = np.round(500.+energy.max(),-3)
         _axis.set_xlim(kwargs_opts['xmin'],kwargs_opts['xmax'])
         _axis.set_ylim(kwargs_opts['ymin'],kwargs_opts['ymax'])
-        _axis.xaxis.set_major_locator(MultipleLocator(1000))
+        # try to make reasonably-spaced xaxis tickmarks.  
+        # if I were clever, I'd do this with a function
+        temperature_range = kwargs_opts['xmax']-kwargs_opts['xmin']
+        if temperature_range <= 10000:
+            _axis.xaxis.set_major_locator(MultipleLocator(1000))
+            _axis.xaxis.set_minor_locator(MultipleLocator(200))
+        elif temperature_range <= 26000:
+            _axis.xaxis.set_major_locator(MultipleLocator(2000))
+            _axis.xaxis.set_minor_locator(MultipleLocator(500))
+        else:
+            _axis.xaxis.set_major_locator(MultipleLocator(6000))
+            _axis.xaxis.set_minor_locator(MultipleLocator(2000))
         _axis.yaxis.set_major_locator(MultipleLocator(1))
-        _axis.xaxis.set_minor_locator(MultipleLocator(200))
         _axis.yaxis.set_minor_locator(MultipleLocator(0.2))
         _axis.tick_params(axis='both',direction='in',which='both')
         _axis.tick_params(axis='both',bottom=True,top=True,left=True,right=True, which='both')
@@ -155,7 +223,9 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
             _axis.grid(b=True,which='minor',axis='both',lw=kwargs_opts['linewidth']/2,
                             color='k',alpha=0.22,linestyle='--')
 
-        _axis.legend(handles,labels)
+        _axis.legend(handles,labels,
+                     bbox_to_anchor=kwargs_opts['bbox_to_anchor'],
+                     loc=kwargs_opts['loc'])
 
     def temperature(self,component,**kwargs):
         """Plot the temperature of hot or cold gas component.
@@ -182,160 +252,6 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
             return self._tool.opr
         self._plot(self._tool.opr,**kwargs)
 
-    def _plot(self,data,**kwargs):
-        '''generic plotting method used by other plot methods'''
-
-        kwargs_plot = {'show' : 'data' # or 'mask' or 'error'
-                      }
-
-        kwargs_opts = {'units' : None,
-                       'image':True,
-                       'colorbar': True,
-                       'contours': True,
-                       'label': False,
-                       'title': None,
-                       'norm': 'simple',
-                       'log': False,
-                       'axis': None
-                       }
-
-        kwargs_contour = {'levels': None,
-                          'colors': ['white'],
-                          'linewidths': 1.0}
-
-        # Merge in any keys the user provided, overriding defaults.
-        kwargs_contour.update(kwargs)
-        kwargs_opts.update(kwargs)
-        kwargs_plot.update(kwargs)
-
-        _data = deepcopy(data)  # default is show the data
-
-        if kwargs_plot['show'] == 'error':
-            _data.data = _data.error
-        # do the log here, because we won't take log of a mask.
-        if kwargs_opts['log']:
-            _data.data = np.log10(_data.data)
-        kwargs_opts.pop('log',None)
-        kwargs.pop('log',None)
-        if kwargs_plot['show'] == 'mask':
-            _data = deepcopy(data)
-            _data.data = _data.mask
-            # can't contour a boolean
-            kwargs_opts['contours'] = False
-
-        if self._tool._modelnaxis == 2 or len(_data.shape)==2:
-            if kwargs_opts['units'] is not None:
-                k = to(kwargs_opts['units'], _data)
-            else:
-                k = _data
-        elif self._tool._modelnaxis == 3:
-            if kwargs_opts['units'] is not None:
-                k = to(kwargs_opts['units'], _data[0,:,:])
-            else:
-                k = _data[0,:,:]
-        else:
-            raise Exception("Unexpected model naxis: %d"%self._tool._modelnaxis)
-
-        km = ma.masked_invalid(k)
-        if getattr(k,"mask",None) is not None:
-            km.mask = np.logical_or(k.mask,km.mask)
-        # make sure nans don't affect the color map
-        min_ = np.nanmin(km)
-        max_ = np.nanmax(km)
-
-        kwargs_imshow = { 'origin': 'lower',
-                          'norm': 'simple',
-                          'stretch': 'linear',
-                          'vmin': min_,
-                          'vmax': max_,
-                          'cmap': 'plasma',
-                          'aspect': 'auto'}
-
-        kwargs_subplot = {'nrows': 1,
-                          'ncols': 1,
-                          'index': 1,
-                          'reset': True,
-                          'constrained_layout': False # this appears to have no effect
-                         }
-
-        # delay merge until min_ and max_ are known
-        kwargs_imshow.update(kwargs)
-        kwargs_imshow['norm']=self._get_norm(kwargs_imshow['norm'],km,
-                                             kwargs_imshow['vmin'],kwargs_imshow['vmax'],
-                                             kwargs_imshow['stretch'])
-
-        kwargs_subplot.update(kwargs)
-        # swap ncols and nrows in figsize to preserve aspect ratio
-        kwargs_subplot['figsize'] = kwargs.get("figsize",(kwargs_subplot["ncols"]*5,kwargs_subplot["nrows"]*5))
-
-        axidx = kwargs_subplot['index']-1
-        if kwargs_opts['axis'] is None:
-            self._figure,self._axis = self._plt.subplots(kwargs_subplot['nrows'],kwargs_subplot['ncols'],
-                                                    figsize=kwargs_subplot['figsize'],
-                                                    subplot_kw={'projection':k.wcs,
-                                                                'aspect':kwargs_imshow['aspect']},
-                                                    constrained_layout=kwargs_subplot['constrained_layout'])
-            _axis = self._axis
-        else:
-            _axis = kwargs_opts['axis']
-
-        # Make sure self._axis is an array because we will index it below.
-        if type(_axis) is not np.ndarray:
-            _axis= np.array([_axis])
-        for a in _axis:
-            a.tick_params(axis='both',direction='in',which='both')
-            # astropy complains if you use axis=both and bottom, top etc.
-            a.tick_params(axis='x',bottom=True,top=True,left=True,right=True, which='both')
-            a.tick_params(axis='y',bottom=True,top=True,left=True,right=True, which='both')
-            if hasattr(a,'coords'):
-                for c in a.coords:
-                    c.display_minor_ticks(True)
-        if kwargs_opts['image']:
-            current_cmap = copy(mcm.get_cmap(kwargs_imshow['cmap']))
-            current_cmap.set_bad(color='white',alpha=1)
-            # suppress errors and warnings about unused keywords
-            #@todo need a better solution for this, it is not scalable.
-            #push onto a stack?
-            for kx in ['units', 'image', 'contours', 'label', 'title','linewidths','levels','nrows','ncols',
-                       'index', 'reset','colors','colorbar','show','yaxis_unit','xaxis_unit','axis',
-                       'constrained_layout','figsize','stretch','legend','markersize','show_fit']:
-                kwargs_imshow.pop(kx,None)
-            # eliminate deprecation warning.  vmin,vmax are passed to Normalization object.
-            if kwargs_opts['norm'] is not None:
-                kwargs_imshow.pop('vmin',None)
-                kwargs_imshow.pop('vmax',None)
-            im=_axis[axidx].imshow(km,**kwargs_imshow)
-            if kwargs_opts['colorbar']:
-                self._wcs_colorbar(im,_axis[axidx])
-
-        if kwargs_opts['contours']:
-            if kwargs_contour['levels'] is None:
-                # Figure out some autolevels
-                kwargs_contour['levels'] = self._autolevels(km,'log')
-
-            # suppress errors and warnings about unused keywords
-            for kx in ['units', 'image', 'contours', 'label', 'title', 'cmap','aspect',
-                       'colorbar','reset', 'nrows', 'ncols', 'index','show','yaxis_unit','axis',
-                       'xaxis_unit','norm','constrained_layout','figsize','stretch','legend','markersize','show_fit']:
-                kwargs_contour.pop(kx,None)
-
-            contourset = _axis[axidx].contour(km, **kwargs_contour)
-            if kwargs_opts['label']:
-                _axis[axidx].clabel(contourset,contourset.levels,inline=True,fmt='%1.1e')
-
-        if kwargs_opts['title'] is not None:
-            #self.figure.subplots_adjust(top=0.95)
-            #self._axis[axidx].set_title(kwargs_opts['title'])
-            # Using ax.set_title causes the title to be cut off.  No amount of
-            # diddling with tight_layout, constrained_layout, subplot adjusting, etc
-            # would affect this.  However using Figure.suptitle seems to work.
-            self.figure.suptitle(kwargs_opts['title'],y=0.95)
-
-        if k.wcs is not None:
-            _axis[axidx].set_xlabel(k.wcs.wcs.lngtyp)
-            _axis[axidx].set_ylabel(k.wcs.wcs.lattyp)
-
-
     def explore(self,data=None,interaction_type="click",**kwargs):
         """Explore the fitted parameters of a map. A user-requested map is displayed in the left panel and in the right panel is the fitted excitation diagram for a point selected by the user.  The user clicks on a point in the left panel and the right panel will update with the excitation diagram for that point.
 
@@ -401,8 +317,10 @@ ExcitationPlot creates excitation diagrams using the results of :class:`~pdrtpy.
                                 norm=True,show_fit=show_fit)
                     #self._axis[0].set_title(f'{position}')
             except Exception as err:
-                pass
-                #self._logfile.write("Exception {0}".format(err))
+                if self._logfile is None:
+                    pass
+                else:
+                    self._logfile.write("Exception {0}".format(err))
 
             #self._logfile.close()
             self._figure.canvas.draw_idle()
