@@ -3,41 +3,41 @@ from copy import deepcopy
 import numpy as np
 import numpy.ma as ma
 
-from astropy.io import fits
+#from astropy.io import fits
 from astropy.io.fits.header import Header
-import astropy.wcs as wcs
+#import astropy.wcs as wcs
 import astropy.units as u
 import astropy.stats as astats
 from astropy.table import Table, Column
 from astropy.nddata import CCDData
 import warnings
-from lmfit import Parameters, Minimizer, minimize, fit_report
+from lmfit import Parameters, Minimizer#, fit_report
 from scipy.interpolate import interpn
-import corner
 from pdrtpy.pbar import get_progress_bar
 
-import cProfile, pstats, io
-from pstats import SortKey
+import cProfile
+import pstats 
+import io
 
 from .toolbase import ToolBase
 from .fitmap import FitMap
 from .. import pdrutils as utils
 from ..modelset import ModelSet
-from ..measurement import Measurement
+#from ..measurement import Measurement
 
 class LineRatioFit(ToolBase):
     """LineRatioFit is a tool to fit observations of intensity ratios to a set of PDR models. It takes as input a set of observations with errors represented as :class:`~pdrtpy.measurement.Measurement` and  :class:`~pdrtpy.modelset.ModelSet` for the models to which the data will be fitted. The observations should be spectral line or continuum intensities.  They can be spatial maps or single pixel values. They should have the same spatial resolution.
-    
-The models to be fit are stored as intensity ratios. The input observations will be use to create ratios that correspond to models. From there a minimization fit is done to determine the density and radiation field that best fit the data.At least 3 observations are needed in order to make at least 2 ratios. With fewer ratios, no fitting can be done. More ratios generally means better determined density and radiation field, assuming the data are consistent with each other.
+
+The models to be fit are stored as intensity ratios. The input observations will be use to create ratios that correspond to models. From there a minimization fit is done to determine the density and radiation field that best fit the data. At least 3 observations are needed in order to make at least 2 ratios. With fewer ratios, no fitting can be done. More ratios generally means better determined density and radiation field, assuming the data are consistent with each other.
 
 Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view the results.
-    
+
 
 :param modelset: The set of PDR models to use for fitting.
 :type modelset: :class:`~pdrtpy.modelset.ModelSet`
 
-:param measurements: Input measurements to be fit.  
-:type measurements: list or dict of :class:`~pdrtpy.measurement.Measurement`. If dict, the keys should be the Measurement *identifiers*.  
+:param measurements: Input measurements to be fit.
+:type measurements: list or dict of :class:`~pdrtpy.measurement.Measurement`. If dict, the keys should be the Measurement *identifiers*.
     """
     def __init__(self,modelset=ModelSet("wk2006",z=1),measurements=None):
         super().__init__() # needed?
@@ -74,11 +74,11 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
     @property
     def fit_result(self):
         '''The result of the fitting procedure which includes fit statistics, variable values and uncertainties, and correlations between variables.  For each pixel, there will be one instance of :class:`lmfit.minimizer.MinimizerResult`.
-        
-        :rtype:  :class:`~pdrtpy.tool.FitMap`      
+
+        :rtype:  :class:`~pdrtpy.tool.FitMap`
         '''
         return self._fitresult
-    
+
     @property
     def modelset(self):
         """The underlying :class:`~pdrtpy.modelset.ModelSet`"""
@@ -87,36 +87,37 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
     @property
     def measurements(self):
         """The stored :class:`measurements <pdrtpy.measurement.Measurement>` as a dictionary with Measurement IDs as keys
-   
+
         :rtype: dict of :class:`~pdrtpy.measurement.Measurement`
         """
         return self._measurements
-    
+
     @property
     def measurementIDs(self):
         '''The stored measurement IDs, which are strings.
 
         :rtype: :class:`dict_keys`
         '''
-         
-        if self._measurements is None: return None
+
+        if self._measurements is None: 
+            return None
         return self._measurements.keys()
 
     @property
     def observed_ratios(self):
         '''The list of the observed line ratios that have been input so far.
- 
+
         :rtype: list of str
         '''
         return list(self._observedratios.keys())
-        
+
     @property
     def ratiocount(self):
         '''The number of ratios that match models available in the current :class:`~pdrtpy.modelset.ModelSet` given the current set of measurements
- 
+
         :rtype: int
         '''
-        # call to  modelset._get_ratio_elements is expensive. So set it once for each run. 
+        # call to  modelset._get_ratio_MA 01938elements is expensive. So set it once for each run.
         if self._ratiocount is None:
             self._ratiocount = self._modelset.ratiocount(self.measurementIDs)
         return self._ratiocount
@@ -138,7 +139,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         return self._radiation_field
 
     def chisq(self,min=False):
-        '''The computed chisquare value(s). 
+        '''The computed chisquare value(s).
 
         :type min: bool
         :param min: If `True` return the minimum reduced :math:`\chi^2`.  In the case of map inputs this will be a spatial map of mininum :math:`\chi^2`.  If `False` with map inputs the entire :math:`\chi^2` hypercube is returned.  If `True` with single pixel inputs, a single value is returned.  If `False` with single pixel inputs, :math:`\chi^2` as a function of density and radiation field is returned.
@@ -152,7 +153,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
 
     def reduced_chisq(self,min=False):
         r'''The computed reduced chisquare value(s).
-        
+
         :type min: bool
         :param min: If `True` return the minimum reduced :math:`\chi_\nu^2`.  In the case of map inputs this will be a spatial map of mininum :math:`\chi_\nu^2`.  If `False` with map inputs the entire :math:`\chi_\nu^2` hypercube is returned.  If `True` with single pixel inputs, a single value is returned.  If `False` with single pixel inputs, :math:`\chi_\nu^2` as a function of density and radiation field is returned.
 
@@ -165,7 +166,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
 
     def _init_measurements(self,m):
         """Initialize the measurements from an input list
-        
+
         :param m: the input list of Measurements
         :type m: list
         """
@@ -177,7 +178,8 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
 
     def _set_model_files_used(self):
         self._model_files_used = dict()
-        if self._measurements is None: return
+        if self._measurements is None: 
+            return
         for x in self._modelset.find_files(self.measurementIDs):
             self._model_files_used[x[0]]=x[1]
 
@@ -187,8 +189,8 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
        return np.all([m.shape == s1 for m in d.values()])
 
     def _check_header(self,kw,value=NotImplemented):
-       """Check to see if any of the given keyword values differ for 
-          the input measurements.  
+       """Check to see if any of the given keyword values differ for
+          the input measurements.
 
           :param kw: the keyword to check
           :type kw: str
@@ -207,7 +209,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
                   in_wcs = True
                else:
                   s1 = h[kw]
-           else: 
+           else:
                s1 = value
            if in_wcs:
                return np.all([m.wcs.to_header()[kw] == s1 for m in d.values()])
@@ -216,19 +218,22 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
        except KeyError:
            print("WARNING: %s keyword not present in all Measurements"%kw)
            return False
-       
+
     def _check_measurement_shapes(self):
-       if self._measurements == None: return False
+       if self._measurements is None: 
+           return False
        return self._check_shapes(self._measurements)
 
     def _check_ratio_shapes(self):
-       if self._observedratios == None: return False
+       if self._observedratios is None: 
+           return False
        return self._check_shapes(self._observedratios)
-    
+
     def _check_model_shapes(self):
-        if self._modelratios == None: return False
-        return self._check_shapes(self._modelratios)
-            
+       if self._modelratios is None: 
+           return False
+       return self._check_shapes(self._modelratios)
+
     def add_measurement(self,m):
         r'''Add a Measurement to internal dictionary used to compute ratios. This measurement may be intensity units (erg :math:`{\rm s}^{-1}` :math:`{\rm cm}^{-2}`) or integrated intensity (K km/s).
 
@@ -241,7 +246,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         else:
             self._init_measurements(m)
         self._set_model_files_used()
-        
+
     def remove_measurement(self,id):
         '''Delete a measurement from the internal dictionary used to compute ratios.
 
@@ -251,18 +256,25 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         '''
         del self._measurements[id]
         self._set_model_files_used()
-    
+
     def read_models(self,unit=u.dimensionless_unscaled):
         """Given a list of measurement IDs, find and open the FITS files that have matching ratios
-        and populate the _modelratios dictionary.  Uses :class:`pdrtpy.measurement.Measurement` as 
-        a storage mechanism. 
+        and populate the _modelratios dictionary.  Uses :class:`pdrtpy.measurement.Measurement` as
+        a storage mechanism.
 
            :param  m: list of measurement IDS (string)
            :type m: list
-           :param unit: units of the data 
+           :param unit: units of the data
            :type unit: string or astropy.Unit
         """
         self._modelratios = self._modelset.get_models(self.measurementIDs,model_type='ratio')
+        if self.ratiocount < 2:
+            msg = f"Not enough ratios.  You need to provide at least 3 observations that can be used to compute 2 ratios that are covered by the ModelSet. From your observations, {self.ratiocount:d} ratio(s)"
+            if self.ratiocount>0:
+                msg += f" {list(self._modelratios.keys())}"
+            msg += " can be computed."
+            raise Exception(msg)
+
         k = utils.firstkey(self._modelratios)
         self._modelnaxis = self._modelratios[k].wcs.naxis
         self._modelshape = np.array(self._modelratios[k].data.shape)
@@ -278,14 +290,14 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
                 try:
                     self.radiation_field_unit = u.Unit(self._modelratios[k].header["CUNIT2"])
                 except KeyError:
-                    raise Exception("Keyword CUNIT2 is required in file %s FITS header to describe units of interstellar radiation field"%thefile)
+                    raise Exception("Keyword CUNIT2 is required in file %s FITS header to describe units of interstellar radiation field"%self._model_files_used[k])
         if not self._check_model_shapes():
             warnings.warn("Trimming all model grids to match H2 grid: log(n) = 1-5, log(G0) = 1-5")
             utils._trim_all_to_H2(self._modelratios)
-            
+
     def _check_compatibility(self):
         """Check that all Measurements are compatible (beams, coordinate systems, shapes) so that the computation make commence.
- 
+
           :raises Exception: if headers and shapes don't match, warns if no beam present
         """
 
@@ -300,7 +312,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
            raise Exception("Beam minor axis (BMIN) of your input Measurements do not match.  Please convolve all maps to the same beam size")
         if not self._check_header("BPA"):
            raise Exception("Beam position angle (BPA) of your input Measurements do not match.  Please convolve all maps to the same beam size")
-           
+
 
         # Check the coordinate systems only if there is more than one pixel
         m1 = self._measurements[utils.firstkey(self._measurements)]
@@ -317,11 +329,11 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         #if not self._check_header("BUNIT") ...
 
     def run(self,**kwargs):
-        '''Run the full computation using all the :class:`observations <pdrtpy.measurement.Measurement>` added.   This will 
-        check compatibility of input observations (e.g., beam parameters, coordinate types, axes lengths) and 
+        '''Run the full computation using all the :class:`observations <pdrtpy.measurement.Measurement>` added.   This will
+        check compatibility of input observations (e.g., beam parameters, coordinate types, axes lengths) and
         raise exceptions if the observations don't match each other.
 
-           :param mask: Indicate how to mask image observations (Measurements) before computing the density 
+           :param mask: Indicate how to mask image observations (Measurements) before computing the density
                         and radiation field. Possible values are:
 
              ['mad', multiplier]   - compute standard deviation using median absolute deviation (astropy.mad_std),
@@ -331,7 +343,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
 
              ['clip', (low,high)]  - mask based on data values, mask out data below low and above high
 
-             ['error', (low,high)] - mask based on uncertainty plane, mask out data where the corresponding error pixel value 
+             ['error', (low,high)] - mask based on uncertainty plane, mask out data where the corresponding error pixel value
                                      is below low or above high
 
                               None - Don't mask data. This is the default.
@@ -344,13 +356,13 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
                 * ’propagate’ : the values returned from userfcn are un-altered
                 * ’omit’ : non-finite values are filtered
            :type nan_policy: str
-               
-           :raises Exception: if no models match the input observations, observations are not compatible, 
+
+           :raises Exception: if no models match the input observations, observations are not compatible,
                               or on unrecognized parameters, or NaN encountered.
         '''
         #@todo global masking for 'data', 'clip', 'error' not entirely useful unless all data/error have same ranges.
         #need something like ['data',['key1':(low,hi), 'key2',(low,hi),...], which is very complicated.
-        # or data/error cut based on histogram 
+        # or data/error cut based on histogram
         kwargs_opts = { 'mask': None,
                         'method': 'leastsq',
                         'nan_policy': 'raise',
@@ -359,7 +371,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
                         'burn': 0,
                         'steps': 1000,
                        # debugging
-                        'test': False, 
+                        'test': False,
                         'profile': False,
         }
         kwargs_opts.update(kwargs)
@@ -382,7 +394,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
 
         # eventually need to check that the maps overlap in real space.
         self._compute_residual()
-        self._minimizer= Minimizer(self._residual_single_pixel, 
+        self._minimizer= Minimizer(self._residual_single_pixel,
                                    params=None, nan_policy=kwargs_opts['nan_policy'])
         #need to pop nan_policy and test so that it does not get passed to Minimzer.minimize()
         kwargs_opts.pop('nan_policy',None)
@@ -395,22 +407,22 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         if profile:
             pr.disable()
             s = io.StringIO()
-            sortby = SortKey.CUMULATIVE
+            sortby = pstats.SortKey.CUMULATIVE
             ps = pstats.Stats(pr, stream=s).sort_stats(sortby)
             ps.print_stats()
             self._stats = s
-            
+
     def _reset_masks(self):
         for m in self._measurements:
             self._measurements[m].mask = deepcopy(self._masks[m])
-            
+
     def _mask_measurements(self,mask):
         ''' Set the mask on the measurements based on noise characteristics.  This is so that
             we don't compute garbage n,G0 where observed ratios are noise divided by noise.
-            
+
            :param mask: Indicate how to mask image observations before computing the density and radiation field. See run()>
         '''
-        if mask is None: 
+        if mask is None:
             return
         if self._measurementnaxis == 0:
             utils.warn(self,"Ignoring 'mask' parameter for single pixel observations")
@@ -445,14 +457,14 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
                 print("Masking %s data where error outside [%.1e,%.1e]"%(k,mask[1][0],mask[1][1]))
         else:
             raise ValueError("Unrecognized mask parameter %s. Valid values are 'mad','data','error'"%mask[0])
-            
+
 
     def _compute_valid_ratios(self):
         '''Compute the valid observed ratio maps for the available model data'''
         if not self._check_measurement_shapes():
             raise Exception("Measurement maps have different dimensions")
 
-        # Note _find_ratio_elements does not handle case of OI+CII/FIR so 
+        # Note _find_ratio_elements does not handle case of OI+CII/FIR so
         # we have to deal with that separately below.
         z = self._modelset._find_ratio_elements(self.measurementIDs)
         self._observedratios = dict()
@@ -491,24 +503,24 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
                 self._observedratios[lab] = aa/bb
                 self._observedratios[lab].meta = deepcopy(bb.header)
                 self._ratioHeader("OI_145+CII_158","FIR",lab)
-                
-    # function to minimize in single-pixel case            
+
+    # function to minimize in single-pixel case
     def _residual_single_pixel(self,params,index):
         parvals = params.valuesdict()
-        mvalue = np.empty(self.ratiocount)  
+        mvalue = np.empty(self.ratiocount)
         dvalue = np.empty(self.ratiocount)
         evalue = np.empty(self.ratiocount)
         i=0
         #@todo create temporary model, data, error as np arrays so the loop over k dictionary key
-        # isn't necessary. 
+        # isn't necessary.
         # OR save residuals from computeDelta so you don't do this twice.
         for k in self._modelratios:
-            mvalue[i] = self._modelratios[k].get(parvals['density'],parvals['radiation_field']) 
+            mvalue[i] = self._modelratios[k].get(parvals['density'],parvals['radiation_field'])
             dvalue[i] = self._observedratios[k].data.flatten()[index]
             evalue[i] = self._observedratios[k].uncertainty.array.flatten()[index]
             i = i+1
         return  (dvalue - mvalue)/evalue
-    
+
     def _residual_multi_pixel(self,params,index):
         # this is currently slower than the 'dumb' way of residual_single_pixel!
         parvals = params.valuesdict()
@@ -517,21 +529,23 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
     def _compute_residual(self):
         '''Compute the residual values from the observed ratios and models
         '''
-        if not self._modelratios: # empty list or None
-            raise Exception("No model data ready.  Was read_models() called?")
-            
-        if self.ratiocount < 2 :
-            raise Exception(f"Not enough ratios.  You need to provide at least 3 observations that can be used to compute 2 ratios that are covered by the ModelSet. From your observations, only {self.ratiocount:d} ratio(s) {list(self._modelratios.keys())} can be computed.")
+
+        if self.ratiocount < 2:
+            msg = f"Not enough ratios.  You need to provide at least 3 observations that can be used to compute 2 ratios that are covered by the ModelSet. From your observations, {self.ratiocount:d} ratio(s)"
+            if self.ratiocount>0:
+                msg += f" {list(self._modelratios.keys())}"
+            msg += " can be computed."
+            raise Exception(msg)
 
         if not self._check_ratio_shapes():
             raise Exception("Observed ratio maps have different dimensions")
-            
+
         self._residual = dict()
         for r in self._observedratios:
             modelpix = self._modelratios[r].data.flatten()
             residuals = list()
             mdata = ma.masked_invalid(self._observedratios[r].value)
-            merror = ma.masked_invalid(self._observedratios[r].error)  
+            merror = ma.masked_invalid(self._observedratios[r].error)
             for pix in modelpix:
                 #optional fractional error correction for log likelihood.
                 _q = (mdata - pix)/merror
@@ -556,7 +570,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
             _qq = np.squeeze(np.reshape(residuals,newshape))
             self._residual[r] = CCDData(_qq,unit="adu",wcs=_wcs,meta=_meta)
         self._fancy_index_residual()
-        
+
     def _fancy_index_residual(self):
         # create a custom numpy array for interpolating the residuals
         # during fitting.
@@ -571,7 +585,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         self._interpvalues = np.stack([self._residual[r].data for r in self._residual.keys()])
         # the flatten the spatial indices, so indices are then [radiation_field,density,pixel]
         self._interpvalues = self._interpvalues.reshape(*self._interpvalues.shape[:-2], -1)
-        
+
     def _interp_resid(self,density,radiation_field,pixel):
         # density and radiation field must be linear not logarithmic values.
         # pixel is 0-based pixel index into flattened map data array.
@@ -579,7 +593,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         for i in range(len(self._residual)):
             vals.append(interpn(self._interpgrid,self._interpvalues,(i,radiation_field,density,pixel)))
         return np.array(vals)
-        
+
     def _compute_chisq(self):
         '''Compute the chi-squared values from observed ratios and models'''
         if self.ratiocount < 2 :
@@ -600,18 +614,18 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         utils.comment(("Reduced Chi-squared (DOF=%d)"%self._dof),self._reduced_chisq)
         self._makehistory(self._chisq)
         self._makehistory(self._reduced_chisq)
-        
+
     def write_chisq(self,chi="chisq.fits",rchi="rchisq.fits",overwrite=True):
         '''Write the chisq and reduced-chisq data to a file
-         
+
            :param chi: FITS file to write the chisq map to.
            :type  chi: str
            :param rchi: FITS file to write the reduced chisq map to.
            :type rchi: str
         '''
         self._chisq.write(chi,overwrite=overwrite,hdu_mask='MASK')
-        self._reduced_chisq.write(rchi,overwrite=overwrite,hdu_mask='MASK')  
-        
+        self._reduced_chisq.write(rchi,overwrite=overwrite,hdu_mask='MASK')
+
     def _refine_density_radiation_field2(self,**kwargs):
         if kwargs['method'] != 'emcee':
             kwargs.pop('steps')
@@ -619,10 +633,10 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
             progress = kwargs.pop("progress",True) # progress bar
         else:
             progress = kwargs.get("progress",False) #keep the progress keyword for emcee, get vs pop
-        # First get the range of density n and radiation field FUV from the 
+        # First get the range of density n and radiation field FUV from the
         # model space, in order to provide them to the Parameters object.
-        # Since the wk2006 H2 models have a smaller model space, 
-        # we have to check if H2 is in one of the models used.          
+        # Since the wk2006 H2 models have a smaller model space,
+        # we have to check if H2 is in one of the models used.
         keys = list(self._modelratios.keys())
         if utils._has_H2(keys):
             # this will get the index for the first modelratio that has H2 in it
@@ -646,6 +660,9 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
             startn = np.nanmean(self._density.value)
             startfuv = np.nanmean(self._radiation_field.value)
         self._fitparam = Parameters()
+    #@todo let user limit these ranges. either they pass in Parameters or
+    # limits = {'density':[low,hi], 'radiation_field':[low,hi]}
+    # in any unit and convert
         self._fitparam.add('density',min=minn,max=maxn,value=startn)
         self._fitparam.add('radiation_field',min=minfuv,max=maxfuv,value=startfuv)
         #self._fitparam.pretty_print()
@@ -668,7 +685,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
             for j in range(self._observedratios[fk].size):
                 #use previous coarse fit as first guess
                 #print("doing pixel ",j, " of ",self._observedratios[fk].size)
-                self._fitparam['density'].value = dflat[j] 
+                self._fitparam['density'].value = dflat[j]
                 self._fitparam['radiation_field'].value = rflat[j]
                 if np.isnan(dflat[j]) or np.isnan(rflat[j]):
                     fmdata[j] = None
@@ -680,12 +697,12 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
                     chi[j] = np.nan
                     rchi[j] = np.nan
                 else:
-                    try: 
+                    try:
                         self._minimizer.userargs=(j,)
                         fmdata[j] = self._minimizer.minimize(params=self._fitparam,**kwargs)
                         #if hasattr(fmdata[j],"success")   ugh.  not guaranteed
                         #if fmdata[j].errorbars:
-                        count = count+1          
+                        count = count+1
                         den[j] = fmdata[j].params['density'].value
                         dene[j] = fmdata[j].params['density'].stderr
                         rf[j] = fmdata[j].params['radiation_field'].value
@@ -732,12 +749,13 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         self._density.uncertainty.array = dene.reshape(dshape)
         self._chisq_min.data = chi.reshape(dshape)
         self._reduced_chisq_min.data = rchi.reshape(dshape)
-        
+
     def _coarse_density_radiation_field(self):
-        '''Compute the best-fit density and radiation field spatial maps 
+        '''Compute the best-fit density and radiation field spatial maps
            by searching for the minimum chi-squared at each spatial pixel.'''
-        if self._chisq is None or self._reduced_chisq is None: return
-        
+        if self._chisq is None or self._reduced_chisq is None: 
+            return
+
         # get the chisq minima of each pixel along the g,n axes
         fk = utils.firstkey(self._modelratios)
         mshape = self._modelratios[fk].shape
@@ -784,7 +802,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
             if self.has_vectors:
                 self._radiation_field.data = g0
             else: #Measurement with image
-                # note this will reshape g0 in radiation_field for us! 
+                # note this will reshape g0 in radiation_field for us!
                 self._radiation_field.data[spatial_idx]=g0
             # We cannot mask nans because numpy does not support writing
             # MaskedArrays to a file. Will get a not implemented error.
@@ -805,7 +823,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
             if self.has_vectors: # Measurement with data vector
                 self._density.data = n
             else: #Measurement with image
-                # note this will reshape g0 in radiation_field for us! 
+                # note this will reshape g0 in radiation_field for us!
                 self._density.data[spatial_idx]=n
                 self._density.data[np.isnan(self._observedratios[fk2])] = np.nan
             # kluge because we dont know how to properly calcultate undertainty on this.
@@ -818,7 +836,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         #self._density.uncertainty.unit =  u.dimensionless_unscaled
 
         #fix the headers
-        self._density_radiation_field_header() 
+        self._density_radiation_field_header()
 
         # now save copies of the 2D min chisquares
         self._chisq_min=deepcopy(self._observedratios[fk2])
@@ -854,7 +872,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         utils.setkey("BUNIT",("Minimum Reduced Chi-squared (DOF=%d)"%self._dof),self._reduced_chisq_min)
         self._makehistory(self._reduced_chisq_min)
         self._makehistory(self._chisq_min)
-                 
+
     def _makehistory(self,image):
         '''Add information to HISTORY keyword indicating how the density and radiation field were computed (measurements given, ratios used)
 
@@ -881,7 +899,7 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         utils.addkey("RATIO",label,self._observedratios[label])
         utils.dataminmax(self._observedratios[label])
         utils.signature(self._observedratios[label])
-        
+
     def _fixheader(self,image):
         '''Put additional axis and header values into an image
 
@@ -911,8 +929,8 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         utils.setkey("CRVAL"+ax2,mod.wcs.wcs.crval[1],image)
         utils.setkey("CRPIX"+ax1,mod.wcs.wcs.crpix[0],image)
         utils.setkey("CRPIX"+ax2,mod.wcs.wcs.crpix[1],image)
-        
-         
+
+
     def _density_radiation_field_header(self):
         '''Common header items in the density and radiation field FITS files'''
         self._density.header.pop('RATIO')
@@ -931,12 +949,12 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
         self._radiation_field.header = Header(self._radiation_field.header)
         self._density._identifier = "H2 Volume Density"
         self._radiation_field._identifier = "Radiation Field"
-        
+
     @property
     def table(self):
         #@TODO: make this work for map data ?
         '''Construct the table of input Measurements, and if the fit has been run, the density, radiation field, and :math:`\chi^2` values
-        
+
         :rtype: :class:`astropy.table.Table`
         '''
         v = self._measurements.values()
@@ -969,29 +987,29 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
 
     #========= Below is deprecated
     def _compute_delta_sq(self):
-        '''Compute the difference-squared values from the observed ratios 
+        '''Compute the difference-squared values from the observed ratios
            and models - multi-pixel version and store in _deltasq member'''
         self._deltasq = self._computeDelta(f=0)
 
     def _computeDelta(self,f):
-        '''Compute the difference-squared values from the observed ratios 
+        '''Compute the difference-squared values from the observed ratios
            and models - multi-pixel version
-           
-           :param f: fractional amount by which the variance is underestimated. 
-           For traditional chi-squared calculation f is zero.  
+
+           :param f: fractional amount by which the variance is underestimated.
+           For traditional chi-squared calculation f is zero.
            For log-likelihood calculation f is positive and less than 1.
            See, e.g. https://emcee.readthedocs.io/en/stable/tutorials/line/#maximum-likelihood-estimation
            :type f: float
         '''
         if not self._modelratios: # empty list or None
             raise Exception("No model data ready.  Was read_models() called?")
-            
+
         if self.ratiocount < 2 :
             raise Exception("Not enough ratios.  You need to provide at least 3 observations that can be used to compute 2 ratios that are covered by the ModelSet. From your observations, only %d ratios can be computed."%self.ratiocount)
 
         if not self._check_ratio_shapes():
             raise Exception("Observed ratio maps have different dimensions")
-            
+
         returnval = dict()
         for r in self._observedratios:
             sz = self._modelratios[r].size
@@ -999,14 +1017,14 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
 
             residuals = list()
             mf = ma.masked_invalid(self._observedratios[r].value)
-            me = ma.masked_invalid(self._observedratios[r].error)  
+            me = ma.masked_invalid(self._observedratios[r].error)
             #frac_error = f*modelpix  # this is actually slower than looping over modelpix
             s2 = me**2
             add_term = 0
             for pix in modelpix:
                 #optional fractional error correction for log likelihood.
                 if f != 0:
-                   #term is actually log(2*pi*s2) but addition of 
+                   #term is actually log(2*pi*s2) but addition of
                    #constant makes no difference in likelihood.
                     frac_error  = f*pix
                     s2 += frac_error**2
@@ -1033,5 +1051,5 @@ Once the fit is done, :class:`~pdrtpy.plot.LineRatioPlot` can be used to view th
             # WCS will be None for single pixel
             _wcs = deepcopy(self._observedratios[r].wcs)
             returnval[r] = CCDData(_qq,unit="adu",wcs=_wcs,meta=_meta)
-    
+
         return returnval
